#!/usr/bin/perl

use strict;
use warnings;

use Test::More tests => 389;

BEGIN {
use_ok('Net::DRI');
use_ok('Net::DRI::Transport');
use_ok('Net::DRI::Exception');
use_ok('Net::DRI::Cache');
use_ok('Net::DRI::Protocol');
use_ok('Net::DRI::Util');
use_ok('Net::DRI::Registry');
use_ok('Net::DRI::Shell');
use_ok('Net::DRI::BaseClass');
use_ok('Net::DRI::Logging');
use_ok('Net::DRI::Logging::Null');
use_ok('Net::DRI::Logging::Files');
use_ok('Net::DRI::Logging::Stderr');
use_ok('Net::DRI::DRD');
use_ok('Net::DRI::DRD::ICANN');
use_ok('Net::DRI::DRD::VeriSign::COM_NET');
use_ok('Net::DRI::DRD::VeriSign::NAME');
use_ok('Net::DRI::DRD::VeriSign::NameStore');
use_ok('Net::DRI::DRD::AFNIC');
use_ok('Net::DRI::DRD::Gandi');
use_ok('Net::DRI::DRD::GDI');
use_ok('Net::DRI::DRD::EURid');
use_ok('Net::DRI::DRD::IIS');
use_ok('Net::DRI::DRD::NASK');
use_ok('Net::DRI::DRD::IENUMAT');
use_ok('Net::DRI::DRD::puntCAT');
use_ok('Net::DRI::DRD::SITA');
use_ok('Net::DRI::DRD::dotMOBI');
use_ok('Net::DRI::DRD::DNSBelgium');
use_ok('Net::DRI::DRD::NicAT');
use_ok('Net::DRI::DRD::DotCooperation');
use_ok('Net::DRI::DRD::Afilias');
use_ok('Net::DRI::DRD::PIR');
use_ok('Net::DRI::DRD::RESTENA');
use_ok('Net::DRI::DRD::Neustar::BIZ');
use_ok('Net::DRI::DRD::DotAsia');
use_ok('Net::DRI::DRD::IUSN');
use_ok('Net::DRI::DRD::auDA');
use_ok('Net::DRI::DRD::Neustar::US');
use_ok('Net::DRI::DRD::OVH');
use_ok('Net::DRI::DRD::BookMyName');
use_ok('Net::DRI::DRD::Nominet');
use_ok('Net::DRI::DRD::DENIC');
use_ok('Net::DRI::DRD::SWITCH');
use_ok('Net::DRI::DRD::RDS');
use_ok('Net::DRI::DRD::VCS');
use_ok('Net::DRI::DRD::SaintVincentGrenadines');
use_ok('Net::DRI::DRD::UHSA');
use_ok('Net::DRI::DRD::BelizeNIC');
use_ok('Net::DRI::DRD::NicLC');
use_ok('Net::DRI::DRD::Datacom');
use_ok('Net::DRI::DRD::doMEn');
use_ok('Net::DRI::DRD::CZNIC');
use_ok('Net::DRI::DRD::Tralliance');
use_ok('Net::DRI::DRD::NORID');
use_ok('Net::DRI::DRD::CGIBR');
use_ok('Net::DRI::DRD::RegistryPro');
use_ok('Net::DRI::DRD::OpenSRS');
use_ok('Net::DRI::DRD::DNSPT');
use_ok('Net::DRI::DRD::CentralNic');
use_ok('Net::DRI::DRD::CoCCA');
use_ok('Net::DRI::DRD::ARNES');
use_ok('Net::DRI::DRD::Domicilium');
use_ok('Net::DRI::DRD::AdamsNames');
use_ok('Net::DRI::DRD::SIDN');
use_ok('Net::DRI::DRD::IITCNR');
use_ok('Net::DRI::DRD::IRegistry');
use_ok('Net::DRI::DRD::CIRA');
use_ok('Net::DRI::DRD::TELEGreenland');
use_ok('Net::DRI::DRD::ISPAPI');
use_ok('Net::DRI::DRD::TRA');
use_ok('Net::DRI::DRD::ZACR');
use_ok('Net::DRI::DRD::TCI');
use_ok('Net::DRI::DRD::SONIC');
use_ok('Net::DRI::DRD::PANDI');
use_ok('Net::DRI::DRD::UPU');
use_ok('Net::DRI::DRD::ICMRegistry');
use_ok('Net::DRI::DRD::Telnic');
use_ok('Net::DRI::Data::Raw');
use_ok('Net::DRI::Data::Hosts');
use_ok('Net::DRI::Data::Changes');
use_ok('Net::DRI::Data::StatusList');
use_ok('Net::DRI::Data::RegistryObject');
use_ok('Net::DRI::Data::Contact');
use_ok('Net::DRI::Data::ContactSet');
use_ok('Net::DRI::Data::Contact::EURid');
use_ok('Net::DRI::Data::Contact::SE');
use_ok('Net::DRI::Data::Contact::PL');
use_ok('Net::DRI::Data::Contact::AFNIC');
use_ok('Net::DRI::Data::Contact::US');
use_ok('Net::DRI::Data::Contact::CAT');
use_ok('Net::DRI::Data::Contact::AERO');
use_ok('Net::DRI::Data::Contact::BE');
use_ok('Net::DRI::Data::Contact::AT');
use_ok('Net::DRI::Data::Contact::COOP');
use_ok('Net::DRI::Data::Contact::LU');
use_ok('Net::DRI::Data::Contact::ASIA');
use_ok('Net::DRI::Data::Contact::Nominet');
use_ok('Net::DRI::Data::Contact::DENIC');
use_ok('Net::DRI::Data::Contact::SWITCH');
use_ok('Net::DRI::Data::Contact::NO');
use_ok('Net::DRI::Data::Contact::BR');
use_ok('Net::DRI::Data::Contact::JOBS');
use_ok('Net::DRI::Data::Contact::FCCN');
use_ok('Net::DRI::Data::Contact::ARNES');
use_ok('Net::DRI::Data::Contact::OpenSRS');
use_ok('Net::DRI::Data::Contact::SIDN');
use_ok('Net::DRI::Data::Contact::CIRA');
use_ok('Net::DRI::Data::Contact::IT');
use_ok('Net::DRI::Data::Contact::COZA');
use_ok('Net::DRI::Data::Contact::TCI');
use_ok('Net::DRI::Data::LanguageTag');
use_ok('Net::DRI::Transport::Socket');
use_ok('Net::DRI::Transport::Dummy');
use_ok('Net::DRI::Transport::Defer');
use_ok('Net::DRI::Protocol::ResultStatus');
use_ok('Net::DRI::Protocol::Message');
use_ok('Net::DRI::Protocol::RRP::Message');
use_ok('Net::DRI::Protocol::RRP::Core::Domain');
use_ok('Net::DRI::Protocol::RRP::Core::Host');
use_ok('Net::DRI::Protocol::RRP::Core::Status');
use_ok('Net::DRI::Protocol::RRP::Core::Session');
use_ok('Net::DRI::Protocol::RRP::Connection');
use_ok('Net::DRI::Protocol::RRP');
use_ok('Net::DRI::Protocol::AFNIC::WS::Domain');
use_ok('Net::DRI::Protocol::AFNIC::WS::Message');
use_ok('Net::DRI::Protocol::AFNIC::WS');
use_ok('Net::DRI::Protocol::EPP');
use_ok('Net::DRI::Protocol::EPP::Message');
use_ok('Net::DRI::Protocol::EPP::Connection');
use_ok('Net::DRI::Protocol::EPP::Util');
use_ok('Net::DRI::Protocol::EPP::Core::Status');
use_ok('Net::DRI::Protocol::EPP::Core::Contact');
use_ok('Net::DRI::Protocol::EPP::Core::Domain');
use_ok('Net::DRI::Protocol::EPP::Core::Host');
use_ok('Net::DRI::Protocol::EPP::Core::Session');
use_ok('Net::DRI::Protocol::EPP::Core::RegistryMessage');
use_ok('Net::DRI::Protocol::EPP::Extensions::GracePeriod');
use_ok('Net::DRI::Protocol::EPP::Extensions::E164');
use_ok('Net::DRI::Protocol::EPP::Extensions::SecDNS');
use_ok('Net::DRI::Protocol::EPP::Extensions::NSgroup');
use_ok('Net::DRI::Protocol::EPP::Extensions::Keygroup');
use_ok('Net::DRI::Protocol::EPP::Extensions::EURid');
use_ok('Net::DRI::Protocol::EPP::Extensions::EURid::Sunrise');
use_ok('Net::DRI::Protocol::EPP::Extensions::EURid::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::EURid::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::EURid::Message');
use_ok('Net::DRI::Protocol::EPP::Extensions::EURid::Registrar');
use_ok('Net::DRI::Protocol::EPP::Extensions::EURid::Notifications');
use_ok('Net::DRI::Protocol::EPP::Extensions::EURid::IDN');
use_ok('Net::DRI::Protocol::EPP::Extensions::EURid::Session');
use_ok('Net::DRI::Protocol::EPP::Extensions::SE');
use_ok('Net::DRI::Protocol::EPP::Extensions::SE::Extensions');
use_ok('Net::DRI::Protocol::EPP::Extensions::PL');
use_ok('Net::DRI::Protocol::EPP::Extensions::PL::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::PL::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::PL::Message');
use_ok('Net::DRI::Protocol::EPP::Extensions::PL::Report');
use_ok('Net::DRI::Protocol::EPP::Extensions::US');
use_ok('Net::DRI::Protocol::EPP::Extensions::US::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::Platforms::COM_NET');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::Platforms::NAME');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::Platforms::NameStore');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::Sync');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::IDNLanguage');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::WhoisInfo');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::NameStore');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::PollLowBalance');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::PollRGP');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::JobsContact');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::WhoWas');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::PremiumDomain');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::Suggestion');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::ClientAttributes');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::TwoFactorAuth');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::ZoneManagement');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::Balance');
use_ok('Net::DRI::Protocol::EPP::Extensions::VeriSign::EmailFwd');
use_ok('Net::DRI::Protocol::EPP::Extensions::AT::Result');
use_ok('Net::DRI::Protocol::EPP::Extensions::AT::IOptions');
use_ok('Net::DRI::Protocol::EPP::Extensions::AT::Message');
use_ok('Net::DRI::Protocol::EPP::Extensions::IENUMAT');
use_ok('Net::DRI::Protocol::EPP::Extensions::CAT');
use_ok('Net::DRI::Protocol::EPP::Extensions::CAT::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::CAT::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::CAT::DefensiveRegistration');
use_ok('Net::DRI::Protocol::EPP::Extensions::AERO');
use_ok('Net::DRI::Protocol::EPP::Extensions::AERO::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::AERO::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::MOBI');
use_ok('Net::DRI::Protocol::EPP::Extensions::MOBI::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::DNSBE');
use_ok('Net::DRI::Protocol::EPP::Extensions::DNSBE::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::DNSBE::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::DNSBE::Message');
use_ok('Net::DRI::Protocol::EPP::Extensions::DNSBE::Notifications');
use_ok('Net::DRI::Protocol::EPP::Extensions::AT');
use_ok('Net::DRI::Protocol::EPP::Extensions::AT::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::AT::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::AT::ATResult');
use_ok('Net::DRI::Protocol::EPP::Extensions::COOP');
use_ok('Net::DRI::Protocol::EPP::Extensions::COOP::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::LU');
use_ok('Net::DRI::Protocol::EPP::Extensions::LU::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::LU::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::LU::Poll');
use_ok('Net::DRI::Protocol::EPP::Extensions::LU::Status');
use_ok('Net::DRI::Protocol::EPP::Extensions::CentralNic');
use_ok('Net::DRI::Protocol::EPP::Extensions::CentralNic::TTL');
use_ok('Net::DRI::Protocol::EPP::Extensions::CentralNic::WebForwarding');
use_ok('Net::DRI::Protocol::EPP::Extensions::CentralNic::Release');
use_ok('Net::DRI::Protocol::EPP::Extensions::CentralNic::Pricing');
use_ok('Net::DRI::Protocol::EPP::Extensions::ASIA');
use_ok('Net::DRI::Protocol::EPP::Extensions::ASIA::CED');
use_ok('Net::DRI::Protocol::EPP::Extensions::ASIA::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::AU');
use_ok('Net::DRI::Protocol::EPP::Extensions::AU::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::E164Validation');
use_ok('Net::DRI::Protocol::EPP::Extensions::E164Validation::RFC5076');
use_ok('Net::DRI::Protocol::EPP::Extensions::Afilias');
use_ok('Net::DRI::Protocol::EPP::Extensions::Afilias::IDNLanguage');
use_ok('Net::DRI::Protocol::EPP::Extensions::Afilias::Restore');
use_ok('Net::DRI::Protocol::EPP::Extensions::Afilias::IPR');
use_ok('Net::DRI::Protocol::EPP::Extensions::Afilias::MaintainerUrl');
use_ok('Net::DRI::Protocol::EPP::Extensions::Afilias::Trademark');
use_ok('Net::DRI::Protocol::EPP::Extensions::Afilias::Message');
use_ok('Net::DRI::Protocol::EPP::Extensions::Afilias::Association');
use_ok('Net::DRI::Protocol::EPP::Extensions::Nominet');
use_ok('Net::DRI::Protocol::EPP::Extensions::Nominet::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::Nominet::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::Nominet::Notifications');
use_ok('Net::DRI::Protocol::EPP::Extensions::SWITCH');
use_ok('Net::DRI::Protocol::EPP::Extensions::CZ');
use_ok('Net::DRI::Protocol::EPP::Extensions::CZ::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::CZ::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::CZ::NSSET');
use_ok('Net::DRI::Protocol::EPP::Extensions::NeuLevel::UIN');
use_ok('Net::DRI::Protocol::EPP::Extensions::NeuLevel::IDNLanguage');
use_ok('Net::DRI::Protocol::EPP::Extensions::NeuLevel::WhoisType');
use_ok('Net::DRI::Protocol::EPP::Extensions::NO');
use_ok('Net::DRI::Protocol::EPP::Extensions::NO::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::NO::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::NO::Host');
use_ok('Net::DRI::Protocol::EPP::Extensions::NO::Message');
use_ok('Net::DRI::Protocol::EPP::Extensions::NO::Result');
use_ok('Net::DRI::Protocol::EPP::Extensions::AFNIC');
use_ok('Net::DRI::Protocol::EPP::Extensions::AFNIC::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::AFNIC::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::AFNIC::Notifications');
use_ok('Net::DRI::Protocol::EPP::Extensions::AFNIC::Status');
use_ok('Net::DRI::Protocol::EPP::Extensions::AFNIC::Session');
use_ok('Net::DRI::Protocol::EPP::Extensions::BR');
use_ok('Net::DRI::Protocol::EPP::Extensions::BR::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::BR::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::PRO');
use_ok('Net::DRI::Protocol::EPP::Extensions::PRO::AV');
use_ok('Net::DRI::Protocol::EPP::Extensions::PRO::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::FCCN');
use_ok('Net::DRI::Protocol::EPP::Extensions::FCCN::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::FCCN::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::ARNES');
use_ok('Net::DRI::Protocol::EPP::Extensions::ARNES::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::ARNES::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::ARNES::Status');
use_ok('Net::DRI::Protocol::EPP::Extensions::IT');
use_ok('Net::DRI::Protocol::EPP::Extensions::IT::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::IT::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::IT::Notifications');
use_ok('Net::DRI::Protocol::EPP::Extensions::IRegistry');
use_ok('Net::DRI::Protocol::EPP::Extensions::SIDN');
use_ok('Net::DRI::Protocol::EPP::Extensions::SIDN::Message');
use_ok('Net::DRI::Protocol::EPP::Extensions::SIDN::Notifications');
use_ok('Net::DRI::Protocol::EPP::Extensions::SIDN::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::SIDN::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::SIDN::Host');
use_ok('Net::DRI::Protocol::EPP::Extensions::CIRA');
use_ok('Net::DRI::Protocol::EPP::Extensions::CIRA::Notifications');
use_ok('Net::DRI::Protocol::EPP::Extensions::CIRA::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::CIRA::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::CIRA::Agreement');
use_ok('Net::DRI::Protocol::EPP::Extensions::CIRA::IDN');
use_ok('Net::DRI::Protocol::EPP::Extensions::ME');
use_ok('Net::DRI::Protocol::EPP::Extensions::ISPAPI');
use_ok('Net::DRI::Protocol::EPP::Extensions::ISPAPI::KeyValue');
use_ok('Net::DRI::Protocol::EPP::Extensions::GL');
use_ok('Net::DRI::Protocol::EPP::Extensions::COZA');
use_ok('Net::DRI::Protocol::EPP::Extensions::COZA::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::COZA::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::CoCCA::IPVerification');
use_ok('Net::DRI::Protocol::EPP::Extensions::TCI');
use_ok('Net::DRI::Protocol::EPP::Extensions::TCI::Contact');
use_ok('Net::DRI::Protocol::EPP::Extensions::TCI::Domain');
use_ok('Net::DRI::Protocol::EPP::Extensions::TCI::Message');
use_ok('Net::DRI::Protocol::EPP::Extensions::TCI::Registrar');
use_ok('Net::DRI::Protocol::EPP::Extensions::CloudRegistry');
use_ok('Net::DRI::Protocol::EPP::Extensions::CloudRegistry::LaunchPhase');
use_ok('Net::DRI::Protocol::EPP::Extensions::IDN');
use_ok('Net::DRI::Protocol::EPP::Extensions::AusRegistry::KeyValue');
use_ok('Net::DRI::Protocol::EPP::Extensions::AusRegistry::IDN');
use_ok('Net::DRI::Protocol::EPP::Extensions::AusRegistry::Variant');
use_ok('Net::DRI::Protocol::EPP::Extensions::AusRegistry::Price');
use_ok('Net::DRI::Protocol::EPP::Extensions::AusRegistry::Sync');
use_ok('Net::DRI::Protocol::EPP::Extensions::ICMRegistry');
use_ok('Net::DRI::Protocol::EPP::Extensions::Telnic');
use_ok('Net::DRI::Protocol::EPP::Extensions::KeyRelay');
use_ok('Net::DRI::Protocol::EPP::Extensions::BIZ');
use_ok('Net::DRI::Protocol::EPP::Extensions::ICANN::MarkSignedMark');
use_ok('Net::DRI::Protocol::EPP::Extensions::ICANN::RegistrarExpirationDate');
use_ok('Net::DRI::Protocol::EPP::Extensions::ResellerInfo');
use_ok('Net::DRI::Protocol::EPP::Extensions::ResellerObject');
use_ok('Net::DRI::Protocol::EPP::Extensions::ServiceMessage');
use_ok('Net::DRI::Protocol::EPP::Extensions::AllocationToken');
use_ok('Net::DRI::Protocol::EPP::Extensions::ChangePoll');
use_ok('Net::DRI::Protocol::EPP::Extensions::CNNIC::Bundling');
use_ok('Net::DRI::Protocol::EPP::Extensions::UnitedTLD::Charge');
use_ok('Net::DRI::Protocol::EPP::Extensions::Reverse');
use_ok('Net::DRI::Protocol::EPP::Extensions::ContactVerification');
use_ok('Net::DRI::Protocol::EPP::Extensions::DomainVerification');
use_ok('Net::DRI::Protocol::DAS');
use_ok('Net::DRI::Protocol::DAS::Message');
use_ok('Net::DRI::Protocol::DAS::Connection');
use_ok('Net::DRI::Protocol::DAS::Domain');
use_ok('Net::DRI::Protocol::DAS::AU');
use_ok('Net::DRI::Protocol::DAS::AU::Message');
use_ok('Net::DRI::Protocol::DAS::AU::Connection');
use_ok('Net::DRI::Protocol::DAS::AU::Domain');
use_ok('Net::DRI::Protocol::DAS::AdamsNames');
use_ok('Net::DRI::Protocol::DAS::AdamsNames::Message');
use_ok('Net::DRI::Protocol::DAS::AdamsNames::Connection');
use_ok('Net::DRI::Protocol::DAS::AdamsNames::Domain');
use_ok('Net::DRI::Protocol::DAS::SIDN');
use_ok('Net::DRI::Protocol::DAS::SIDN::Message');
use_ok('Net::DRI::Protocol::DAS::SIDN::Connection');
use_ok('Net::DRI::Protocol::DAS::SIDN::Domain');
use_ok('Net::DRI::Protocol::Whois');
use_ok('Net::DRI::Protocol::Whois::Message');
use_ok('Net::DRI::Protocol::Whois::Connection');
use_ok('Net::DRI::Protocol::Whois::Domain::common');
use_ok('Net::DRI::Protocol::Whois::Domain::COM');
use_ok('Net::DRI::Protocol::Whois::Domain::ORG');
use_ok('Net::DRI::Protocol::Whois::Domain::AERO');
use_ok('Net::DRI::Protocol::Whois::Domain::INFO');
use_ok('Net::DRI::Protocol::Whois::Domain::EU');
use_ok('Net::DRI::Protocol::Whois::Domain::BIZ');
use_ok('Net::DRI::Protocol::Whois::Domain::MOBI');
use_ok('Net::DRI::Protocol::Whois::Domain::NAME');
use_ok('Net::DRI::Protocol::Whois::Domain::LU');
use_ok('Net::DRI::Protocol::Whois::Domain::WS');
use_ok('Net::DRI::Protocol::Whois::Domain::SE');
use_ok('Net::DRI::Protocol::Whois::Domain::CAT');
use_ok('Net::DRI::Protocol::Whois::Domain::AT');
use_ok('Net::DRI::Protocol::Whois::Domain::TRAVEL');
use_ok('Net::DRI::Protocol::Whois::Domain::PT');
use_ok('Net::DRI::Protocol::OVH::WS');
use_ok('Net::DRI::Protocol::OVH::WS::Connection');
use_ok('Net::DRI::Protocol::OVH::WS::Message');
use_ok('Net::DRI::Protocol::OVH::WS::Account');
use_ok('Net::DRI::Protocol::OVH::WS::Domain');
use_ok('Net::DRI::Protocol::BookMyName::WS');
use_ok('Net::DRI::Protocol::BookMyName::WS::Message');
use_ok('Net::DRI::Protocol::BookMyName::WS::Account');
use_ok('Net::DRI::Protocol::BookMyName::WS::Domain');
use_ok('Net::DRI::Protocol::Gandi::WS');
use_ok('Net::DRI::Protocol::Gandi::WS::Message');
use_ok('Net::DRI::Protocol::Gandi::WS::Account');
use_ok('Net::DRI::Protocol::Gandi::WS::Domain');
use_ok('Net::DRI::Protocol::RRI');
use_ok('Net::DRI::Protocol::RRI::Connection');
use_ok('Net::DRI::Protocol::RRI::Contact');
use_ok('Net::DRI::Protocol::RRI::Message');
use_ok('Net::DRI::Protocol::RRI::Session');
use_ok('Net::DRI::Protocol::RRI::Domain');
use_ok('Net::DRI::Protocol::RRI::RegistryMessage');
use_ok('Net::DRI::Protocol::OpenSRS::XCP');
use_ok('Net::DRI::Protocol::OpenSRS::XCP::Account');
use_ok('Net::DRI::Protocol::OpenSRS::XCP::Domain');
use_ok('Net::DRI::Protocol::OpenSRS::XCP::Session');
use_ok('Net::DRI::Protocol::OpenSRS::XCP::Message');
use_ok('Net::DRI::Protocol::OpenSRS::XCP::Host');
use_ok('Net::DRI::Protocol::IRIS');
use_ok('Net::DRI::Protocol::IRIS::Message');
use_ok('Net::DRI::Protocol::IRIS::Core');
use_ok('Net::DRI::Protocol::IRIS::DCHK::Domain');
use_ok('Net::DRI::Protocol::IRIS::DCHK::Status');
use_ok('Net::DRI::Protocol::IRIS::XCP');
use_ok('Net::DRI::Protocol::AdamsNames::WS');
use_ok('Net::DRI::Protocol::AdamsNames::WS::Connection');
use_ok('Net::DRI::Protocol::AdamsNames::WS::Message');
use_ok('Net::DRI::Protocol::AdamsNames::WS::Domain');
}

exit 0;
